import pqb
from setuptools import setup

setup(name='pqb',
      version=pqb.__version__,
      description='Simple SQL Query Builder',
      url='http://github.com/josegomezr/pqb',
      author='Jose Gomez',
      author_email='1josegomezr@gmail.com',
      license='MIT',
      packages=['pqb'],
      zip_safe=False
    )